{+-----------------------------------------------------------------------------+
 | Class:       TmwCSSSyn
 | Created:     1999-11
 | Last change: 2000-01-21
 | Author:      Tony De Buys [tony@lad.co.za]
 | Description: Cascading Stylesheets highlighter
 | Version:     0.5
 | Copyright (c) 1999 Tony De Buys
 | All rights reserved.
 | Portions Copyright Martin Waldenburg.
 |
 | Version history: see version.rtf
 |
 +----------------------------------------------------------------------------+}

unit mwCSSSyn;

{$I mwEdit.inc}

interface

uses
  SysUtils, Windows, Messages, Classes, Controls, Graphics, Registry,
  mwHighlighter;

Type
  TtkTokenKind = (
    tkComment,
    tkIdentifier,
    tkKey,
    tkNull,
    tkNumber,
    tkSpace,
    tkString,
    tkSymbol,
    tkUnknown);

  TRangeState = (rsUnknown, rsCStyle);

  TProcTableProc = procedure of object;

  PIdentFuncTableFunc = ^TIdentFuncTableFunc;
  TIdentFuncTableFunc = function: TtkTokenKind of object;

type
  TmwCSSSyn = class(TmwCustomHighLighter)
  private
    fRange: TRangeState;
    fLine: PChar;
    fLineNumber: Integer;
    fProcTable: array[#0..#255] of TProcTableProc;
    Run: LongInt;
    fStringLen: Integer;
    fToIdent: PChar;
    fTokenPos: Integer;
    FTokenID: TtkTokenKind;
    fIdentFuncTable: array[0..138] of TIdentFuncTableFunc;
    fCommentAttri: TmwHighLightAttributes;
    fIdentifierAttri: TmwHighLightAttributes;
    fKeyAttri: TmwHighLightAttributes;
    fNumberAttri: TmwHighLightAttributes;
    fSpaceAttri: TmwHighLightAttributes;
    fStringAttri: TmwHighLightAttributes;
    fSymbolAttri: TmwHighLightAttributes;
    function KeyHash(ToHash: PChar): Integer;
    function KeyComp(const aKey: String): Boolean;
    function Func15: TtkTokenKind;
    function Func18: TtkTokenKind;
    function Func36: TtkTokenKind;
    function Func40: TtkTokenKind;
    function Func43: TtkTokenKind;
    function Func49: TtkTokenKind;
    function Func51: TtkTokenKind;
    function Func55: TtkTokenKind;
    function Func57: TtkTokenKind;
    function Func59: TtkTokenKind;
    function Func62: TtkTokenKind;
    function Func63: TtkTokenKind;
    function Func64: TtkTokenKind;
    function Func66: TtkTokenKind;
    function Func69: TtkTokenKind;
    function Func72: TtkTokenKind;
    function Func85: TtkTokenKind;
    function Func90: TtkTokenKind;
    function Func96: TtkTokenKind;
    function Func116: TtkTokenKind;
    function Func117: TtkTokenKind;
    function Func138: TtkTokenKind;
    procedure AsciiCharProc;
    procedure CRProc;
    procedure CStyleCommentProc;
    procedure IdentProc;
    procedure IntegerProc;
    procedure LFProc;
    procedure NullProc;
    procedure NumberProc;
    procedure RoundOpenProc;
    procedure SlashProc;
    procedure SpaceProc;
    procedure StringProc;
    procedure UnknownProc;
    function AltFunc: TtkTokenKind;
    procedure InitIdent;
    function IdentKind(MayBe: PChar): TtkTokenKind;
    procedure MakeMethodTables;
  protected
    function GetIdentChars: TIdentChars; override;
  public
    class function GetCapability: THighlighterCapability; override;             //gp 2000-01-20
    class function GetLanguageName: string; override;                           //gp 2000-01-20
  public
    constructor Create(AOwner: TComponent); override;
    function GetDefaultAttribute(Index: integer): TmwHighLightAttributes;       //mh 2000-01-17
      override;
    function GetEOL: Boolean; override;
    function GetRange: Pointer; override;
    function GetTokenID: TtkTokenKind;
    procedure SetLine(NewValue: String; LineNumber: Integer); override;
    procedure ExportNext; override;
    procedure SetLineForExport(NewValue: String); override;
    function GetToken: String; override;
    function GetTokenAttribute: TmwHighLightAttributes; override;
    function GetTokenKind: integer; override;
    function GetTokenPos: Integer; override;
    procedure Next; override;
    procedure SetRange(Value: Pointer); override;
    procedure ReSetRange; override;
    property IdentChars;
  published
    property CommentAttri: TmwHighLightAttributes read fCommentAttri
      write fCommentAttri;
    property IdentifierAttri: TmwHighLightAttributes read fIdentifierAttri
      write fIdentifierAttri;
    property KeyAttri: TmwHighLightAttributes read fKeyAttri write fKeyAttri;
    property NumberAttri: TmwHighLightAttributes read fNumberAttri
      write fNumberAttri;
    property SpaceAttri: TmwHighLightAttributes read fSpaceAttri
      write fSpaceAttri;
    property StringAttri: TmwHighLightAttributes read fStringAttri
      write fStringAttri;
    property SymbolAttri: TmwHighLightAttributes read fSymbolAttri
      write fSymbolAttri;
  end;

procedure Register;

implementation

uses
  mwExport, mwLocalStr;

procedure Register;
begin
  RegisterComponents(MWS_HighlightersPage, [TmwCSSSyn]);
end;

var
  Identifiers: array[#0..#255] of ByteBool;
  mHashTable: array[#0..#255] of Integer;

procedure MakeIdentTable;
var
  I, J: Char;
begin
  for I := #0 to #255 do
  begin
    Case I of
      '_', '0'..'9', 'a'..'z', 'A'..'Z': Identifiers[I] := True;
      else Identifiers[I] := False;
    end;
    J := UpCase(I);
    Case I in ['_', 'A'..'Z', 'a'..'z'] of
      True: mHashTable[I] := Ord(J) - 64
      else mHashTable[I] := 0;
    end;
  end;
end;

procedure TmwCSSSyn.InitIdent;
var
  I: Integer;
  pF: PIdentFuncTableFunc;
begin
  pF := PIdentFuncTableFunc(@fIdentFuncTable);
  for I := Low(fIdentFuncTable) to High(fIdentFuncTable) do begin
    pF^ := AltFunc;
    Inc(pF);
  end;
  fIdentFuncTable[15] := Func15;
  fIdentFuncTable[18] := Func18;
  fIdentFuncTable[36] := Func36;
  fIdentFuncTable[40] := Func40;
  fIdentFuncTable[43] := Func43;
  fIdentFuncTable[49] := Func49;
  fIdentFuncTable[51] := Func51;
  fIdentFuncTable[55] := Func55;
  fIdentFuncTable[57] := Func57;
  fIdentFuncTable[59] := Func59;
  fIdentFuncTable[62] := Func62;
  fIdentFuncTable[63] := Func63;
  fIdentFuncTable[64] := Func64;
  fIdentFuncTable[66] := Func66;
  fIdentFuncTable[69] := Func69;
  fIdentFuncTable[72] := Func72;
  fIdentFuncTable[85] := Func85;
  fIdentFuncTable[90] := Func90;
  fIdentFuncTable[96] := Func96;
  fIdentFuncTable[116] := Func116;
  fIdentFuncTable[117] := Func117;
  fIdentFuncTable[138] := Func138;
end;

function TmwCSSSyn.KeyHash(ToHash: PChar): Integer;
begin
  Result := 0;
  while ToHash^ in ['_', '0'..'9', 'a'..'z', 'A'..'Z'] do
  begin
    inc(Result, mHashTable[ToHash^]);
    inc(ToHash);
  end;
  fStringLen := ToHash - fToIdent;
end;

function TmwCSSSyn.KeyComp(const aKey: String): Boolean;
var
  I: Integer;
  Temp: PChar;
begin
  Temp := fToIdent;
  if Length(aKey) = fStringLen then
  begin
    Result := True;
    for i := 1 to fStringLen do
    begin
      if mHashTable[Temp^] <> mHashTable[aKey[i]] then
      begin
        Result := False;
        break;
      end;
      inc(Temp);
    end;
  end else Result := False;
end;

function TmwCSSSyn.Func15: TtkTokenKind;
begin
  if KeyComp('face') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func18: TtkTokenKind;
begin
  if KeyComp('em') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func36: TtkTokenKind;
begin
  if KeyComp('pt') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func40: TtkTokenKind;
begin
  if KeyComp('px') then Result := tkKey else
    if KeyComp('line') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func43: TtkTokenKind;
begin
  if KeyComp('left') then Result := tkKey else
    if KeyComp('align') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func49: TtkTokenKind;
begin
  if KeyComp('shape') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func51: TtkTokenKind;
begin
  if KeyComp('top') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func55: TtkTokenKind;
begin
  if KeyComp('padding') then Result := tkKey else
    if KeyComp('font') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func57: TtkTokenKind;
begin
  if KeyComp('height') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func59: TtkTokenKind;
begin
  if KeyComp('size') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func62: TtkTokenKind;
begin
  if KeyComp('right') then Result := tkKey else
    if KeyComp('margin') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func63: TtkTokenKind;
begin
  if KeyComp('color') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func64: TtkTokenKind;
begin
  if KeyComp('width') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func66: TtkTokenKind;
begin
  if KeyComp('family') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func69: TtkTokenKind;
begin
  if KeyComp('text') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func72: TtkTokenKind;
begin
  if KeyComp('weight') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func85: TtkTokenKind;
begin
  if KeyComp('bottom') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func90: TtkTokenKind;
begin
  if KeyComp('vertical') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func96: TtkTokenKind;
begin
  if KeyComp('background') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func116: TtkTokenKind;
begin
  if KeyComp('overflow') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func117: TtkTokenKind;
begin
  if KeyComp('position') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.Func138: TtkTokenKind;
begin
  if KeyComp('horizontal') then Result := tkKey else Result := tkIdentifier;
end;

function TmwCSSSyn.AltFunc: TtkTokenKind;
begin
  Result := tkIdentifier;
end;

function TmwCSSSyn.IdentKind(MayBe: PChar): TtkTokenKind;
var
  HashKey: Integer;
begin
  fToIdent := MayBe;
  HashKey := KeyHash(MayBe);
  if HashKey < 139 then Result := fIdentFuncTable[HashKey] else Result := tkIdentifier;
end;

procedure TmwCSSSyn.MakeMethodTables;
var
  I: Char;
begin
  for I := #0 to #255 do
    case I of
      '{', '}': fProcTable[I] := AsciiCharProc;
      #13: fProcTable[I] := CRProc;
      'A'..'Z', 'a'..'z', '_': fProcTable[I] := IdentProc;
      '#', '$': fProcTable[I] := IntegerProc;
      #10: fProcTable[I] := LFProc;
      #0: fProcTable[I] := NullProc;
      '0'..'9': fProcTable[I] := NumberProc;
      ')', '(': fProcTable[I] := RoundOpenProc;
      '/': fProcTable[I] := SlashProc;
      #1..#9, #11, #12, #14..#32: fProcTable[I] := SpaceProc;
      #39: fProcTable[I] := StringProc;
      else fProcTable[I] := UnknownProc;
    end;
end;

constructor TmwCSSSyn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fCommentAttri := TmwHighLightAttributes.Create(MWS_AttrComment);
  fCommentAttri.Style := [fsItalic];
  AddAttribute(fCommentAttri);
  fIdentifierAttri := TmwHighLightAttributes.Create(MWS_AttrIdentifier);
  AddAttribute(fIdentifierAttri);
  fKeyAttri := TmwHighLightAttributes.Create(MWS_AttrKey);
  fKeyAttri.Style := [fsBold];
  AddAttribute(fKeyAttri);
  fNumberAttri := TmwHighLightAttributes.Create(MWS_AttrNumber);
  AddAttribute(fNumberAttri);
  fSpaceAttri := TmwHighLightAttributes.Create(MWS_AttrSpace);
  AddAttribute(fSpaceAttri);
  fStringAttri := TmwHighLightAttributes.Create(MWS_AttrString);
  AddAttribute(fStringAttri);
  fSymbolAttri := TmwHighLightAttributes.Create(MWS_AttrSymbol);
  AddAttribute(fSymbolAttri);
  SetAttributesOnChange(DefHighlightChange);
  InitIdent;
  MakeMethodTables;
  fDefaultFilter := MWS_FilterCSS;
  fRange := rsUnknown;
end;

procedure TmwCSSSyn.SetLine(NewValue: String; LineNumber: Integer);
begin
  fLine := PChar(NewValue);
  Run := 0;
  fLineNumber := LineNumber;
  Next;
end;

procedure TmwCSSSyn.AsciiCharProc;
begin
  fTokenID := tkString;
  inc(Run);
  while FLine[Run] in ['0'..'9'] do inc(Run);
end;

procedure TmwCSSSyn.CRProc;
begin
  fTokenID := tkSpace;
  inc(Run);
  if fLine[Run] = #10 then inc(Run);
end;

procedure TmwCSSSyn.CStyleCommentProc;
begin
  if fLine[Run] = #0 then
    fTokenID := tkNull
  else begin
    fTokenID := tkComment;
    repeat
      if (fLine[Run] = '*') and (fLine[Run + 1] = '/') then begin
        fRange := rsUnKnown;
        inc(Run, 2);
        break;
      end;
      inc(Run);
    until fLine[Run] in [#0, #10, #13];
  end;
end;

procedure TmwCSSSyn.IdentProc;
begin
  fTokenID := IdentKind((fLine + Run));
  inc(Run, fStringLen);
  while Identifiers[fLine[Run]] do inc(Run);
end;

procedure TmwCSSSyn.IntegerProc;
begin
  inc(Run);
  fTokenID := tkNumber;
  while FLine[Run] in ['0'..'9', 'A'..'F', 'a'..'f'] do inc(Run);
end;

procedure TmwCSSSyn.LFProc;
begin
  fTokenID := tkSpace;
  inc(Run);
end;

procedure TmwCSSSyn.NullProc;
begin
  fTokenID := tkNull;
end;

procedure TmwCSSSyn.NumberProc;
begin
  inc(Run);
  fTokenID := tkNumber;
  while FLine[Run] in ['0'..'9', '.', 'e', 'E'] do
  begin
    case FLine[Run] of
      '.':
        if FLine[Run + 1] = '.' then break;
    end;
    inc(Run);
  end;
end;

procedure TmwCSSSyn.RoundOpenProc;
begin
  inc(Run);
  FTokenID := tkSymbol;
end;

procedure TmwCSSSyn.SlashProc;
begin
  inc(Run);
  if fLine[Run] = '*' then begin
    fTokenID := tkComment;
    fRange := rsCStyle;
    inc(Run);
    if not (fLine[Run] in [#0, #10, #13]) then
      CStyleCommentProc;
  end else
    fTokenID := tkSymbol;
end;

procedure TmwCSSSyn.SpaceProc;
begin
  inc(Run);
  fTokenID := tkSpace;
  while FLine[Run] in [#1..#9, #11, #12, #14..#32] do inc(Run);
end;

procedure TmwCSSSyn.StringProc;
begin
  fTokenID := tkString;
  if (FLine[Run + 1] = #39) and (FLine[Run + 2] = #39) then inc(Run, 2);
  repeat
    case FLine[Run] of
      #0, #10, #13: break;
    end;
    inc(Run);
  until FLine[Run] = #39;
  if FLine[Run] <> #0 then inc(Run);
end;

procedure TmwCSSSyn.UnknownProc;
begin
  inc(Run);
  fTokenID := tkUnknown;
end;

procedure TmwCSSSyn.Next;
begin
  fTokenPos := Run;
  if fRange = rsCStyle then
    CStyleCommentProc
  else
    fProcTable[fLine[Run]];
end;

{begin}                                                                         //mh 2000-01-17
function TmwCSSSyn.GetDefaultAttribute(Index: integer): TmwHighLightAttributes;
begin
  case Index of
    MW_DEFATTR_COMMENT: Result := fCommentAttri;
    MW_DEFATTR_KEYWORD: Result := fKeyAttri;
    MW_DEFATTR_WHITESPACE: Result := fSpaceAttri;
    else Result := nil;
  end;
end;
{end}                                                                           //mh 2000-01-17

function TmwCSSSyn.GetEOL: Boolean;
begin
  Result := fTokenID = tkNull;
end;

function TmwCSSSyn.GetRange: Pointer;
begin
  Result := Pointer(fRange);
end;

function TmwCSSSyn.GetToken: String;
var
  Len: LongInt;
begin
  Len := Run - fTokenPos;
  SetString(Result, (FLine + fTokenPos), Len);
end;

function TmwCSSSyn.GetTokenID: TtkTokenKind;
begin
  Result := fTokenId;
end;

function TmwCSSSyn.GetTokenAttribute: TmwHighLightAttributes;
begin
  case GetTokenID of
    tkComment: Result := fCommentAttri;
    tkIdentifier: Result := fIdentifierAttri;
    tkKey: Result := fKeyAttri;
    tkNumber: Result := fNumberAttri;
    tkSpace: Result := fSpaceAttri;
    tkString: Result := fStringAttri;
    tkSymbol: Result := fSymbolAttri;
    tkUnknown: Result := fIdentifierAttri;
    else Result := nil;
  end;
end;

function TmwCSSSyn.GetTokenKind: integer;
begin
  Result := Ord(fTokenId);
end;

function TmwCSSSyn.GetTokenPos: Integer;
begin
  Result := fTokenPos;
end;

procedure TmwCSSSyn.ReSetRange;
begin
  fRange := rsUnknown;
end;

procedure TmwCSSSyn.SetRange(Value: Pointer);
begin
  fRange := TRangeState(Value);
end;

function TmwCSSSyn.GetIdentChars: TIdentChars;
begin
  Result := ['_', '0'..'9', 'a'..'z', 'A'..'Z'];
end;

class function TmwCSSSyn.GetLanguageName: string;                               //gp 2000-01-20
begin
  Result := MWS_LangCSS;
end;

class function TmwCSSSyn.GetCapability: THighlighterCapability;                 //gp 2000-01-20
begin
  Result := inherited GetCapability + [hcExportable];
end;  

procedure TmwCSSSyn.SetLineForExport(NewValue: String);
begin
  fLine := PChar(NewValue);
  Run := 0;
  ExportNext;
end;

procedure TmwCSSSyn.ExportNext;
begin
  Next;
  if Assigned(Exporter) then
    with TmwCustomExport(Exporter) do
      case GetTokenID of
        tkComment: FormatToken(GetToken, fCommentAttri, True, False);
        tkIdentifier: FormatToken(GetToken, fIdentifierAttri, False, False);
        tkKey: FormatToken(GetToken, fKeyAttri, False, False);
        // Needed to catch line breaks
        tkNull: FormatToken('', nil, False, False);
        tkNumber: FormatToken(GetToken, fNumberAttri, False, False);
        tkSpace: FormatToken(GetToken, fSpaceAttri, False, True);
        tkString: FormatToken(GetToken, fStringAttri, True, False);
        tkSymbol: FormatToken(GetToken, fSymbolAttri, True, False);
    end;
end;

initialization
  MakeIdentTable;
  RegisterPlaceableHighlighter(TmwCSSSyn);
end.
