{+-----------------------------------------------------------------------------+
 | Created:     31.03.99
 | Last change: 2000-01-27
 | Author:      Olivier Deckmyn
 | Description: This simple component implements editing rules to apply to a
 |              python source file. Python has a unusual way to mark blocks
 |              (like begin/end in pascal) : it uses indentation. So the rule is
 |              after a ":" and a line break, we have to indent once.
 |
 | Version:     0.14 (see version.rtf for version history)
 | Copyright (c) 1998 Olivier Deckmyn
 | No rights reserved.
 |
 | Thanks to: Martin Waldenburg (as usual ;-) ), Primoz Gabrijelcic
 +----------------------------------------------------------------------------+}

unit odPythonBehaviour;

{$I mwEdit.inc}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  mwCustomEdit, mwKeyCmds;

type
  TodPythonBehaviour = class(TComponent)
  private
    FEditor: TmwCustomEdit;
{begin}                                                                         //sva 2000-01-27
//    FFormerKeyPress: TKeyPressEvent;
//    FFormProcessUserCommand : TProcessCommandEvent;
    FFormerProcessCommand: TProcessCommandEvent;
    fIndent: integer;
    fEnabled: Boolean;
    procedure SetEnabled(Value: Boolean); virtual;
  protected
    procedure SetEditor(Value: TmwCustomEdit); virtual;
//    procedure doKeyPress(Sender: TObject; var Key: Char); virtual;
    procedure DoProcessCommand(Sender: TObject; var Command: TmwEditorCommand;
      var AChar: Char; Data: pointer); virtual;
//    procedure DoProcessUserCommand(Sender: TObject;
//      var Command: TmwEditorCommand; var AChar: Char; Data: Pointer); virtual;
{end}                                                                           //sva 2000-01-27
  public
    procedure AttachFormerEvents;
    constructor Create(AOwner: TComponent); override;
    procedure Loaded; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation);
      override;
  published
    property Editor: TmwCustomEdit read FEditor write SetEditor;
    property Indent: integer read fIndent write fIndent default 4;
    property Enabled: Boolean read FEnabled write SetEnabled default True;      //sva 2000-01-27
  end;

procedure Register;

implementation

uses
  mwLocalStr;

procedure Register;
begin
  RegisterComponents(MWS_ComponentsPage, [TodPythonBehaviour]);
end;

procedure TodPythonBehaviour.AttachFormerEvents;
begin
  if Assigned(FEditor) then
  begin
{begin}                                                                         //sva 2000-01-27
(*
    FFormerKeyPress := FEditor.OnKeyPress;
    FFormProcessUserCommand := FEditor.OnProcessUserCommand;
    FEditor.OnKeyPress := doKeyPress;
    FEditor.OnProcessUserCommand := doProcessUserCommand;
*)
    FFormerProcessCommand := FEditor.OnProcessCommand;
    FEditor.OnProcessCommand := DoProcessCommand;
{end}                                                                           //sva 2000-01-27
  end;
end;

constructor TodPythonBehaviour.Create(aOwner: TComponent);
begin
  inherited Create(AOwner);
  fEnabled := TRUE;
  fIndent := 4;
end;

{begin}                                                                         //sva 2000-01-27
procedure TodPythonBehaviour.DoProcessCommand(Sender: TObject;
  var Command: TmwEditorCommand; var AChar: Char; Data: pointer);
var
  i: integer;
  Line: string;
begin
  if Assigned(FFormerProcessCommand) then
    FFormerProcessCommand(Self, Command, AChar, Data);
  if fEnabled and Assigned(FEditor) and (Command = ecLineBreak) then
  try
    Line := Trim(FEditor.LineText);
    if Line[Length(Line)] = ':' then
    begin
      Command := ecNone;
      fEnabled := FALSE;
      try
        FEditor.CommandProcessor(ecLineBreak, #0, nil);
      finally
        fEnabled := TRUE;
      end;
      for i := 1 to fIndent do
        FEditor.CommandProcessor(ecChar, #32, nil);
    end;
  except
  end;
end;
{end}                                                                           //sva 2000-01-27

procedure TodPythonBehaviour.Loaded;
begin
  inherited Loaded;
  if not (csDesigning in ComponentState) then
    AttachFormerEvents;
end;

procedure TodPythonBehaviour.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) and (aComponent = FEditor) then
    FEditor := nil;
end;

procedure TodPythonBehaviour.SetEditor(Value: TmwCustomEdit);
begin
  if FEditor <> Value then
  begin
    // First restore the former event handlers, if any
    if (ComponentState * [csDesigning, csLoading] = []) and Assigned(FEditor)
    then begin
{begin}                                                                         //sva 2000-01-27
(*
      if assigned(FFormerKeyPress) then
      begin
        FEditor.OnKeypress := FFormerKeyPress;
        FFormerKeyPress := nil;
      end;
      if assigned(FFormProcessUserCommand) then
      begin
        FEditor.OnProcessUserCommand := FFormProcessUserCommand;
        FFormProcessUserCommand := nil;
      end;
*)
      if Assigned(FFormerProcessCommand) then
      begin
        FEditor.OnProcessCommand := FFormerProcessCommand;
        FFormerProcessCommand := nil;
      end;
{end}                                                                           //sva 2000-01-27
    end;
    // Set the new editor
    FEditor := Value;
    // Attach the new event handlers
    if (ComponentState * [csDesigning, csLoading] = []) then
      AttachFormerEvents;
  end;
end;  // SetEditor

procedure TodPythonBehaviour.SetEnabled(Value: Boolean);
begin
  fEnabled := Value;
end;

end.
