{+-----------------------------------------------------------------------------+
 | Class:       TwmMultiSyn
 | Created:     not known
 | Last change: 2000-01-17
 | Author:      Willo van der Merwe [willo@wack.co.za]
 | Description: General multi highliter
 | Version:     0.51
 | Copyright (c) 1999 Willo van der Merwe
 | All rights reserved.
 | Portions Copyright Martin Waldenburg.
 |
 | Version history: see version.rtf
 |
 +----------------------------------------------------------------------------+}

unit wmMultiSyn;

{$I mwEdit.inc}

interface

uses
  Classes, mwHighlighter;

type
  TgmScheme = class( TCollectionItem )
  private
    FEndExpr: string;
    FStartExpr: string;
    FHighlighter: TmwCustomHighLighter;
    procedure SetEndExpr(const Value: string);
    procedure SetHighlighter(const Value: TmwCustomHighLighter);
    procedure SetStartExpr(const Value: string);
  published
    property StartExpr : string read FStartExpr write SetStartExpr;
    property EndExpr : string read FEndExpr write SetEndExpr;
    property Highlighter : TmwCustomHighLighter read FHighlighter
      write SetHighlighter;
  end;
  TgmSchemeClass = class of TgmScheme;

  TwmMultiSyn = class;

  TgmSchemes = class( TCollection )
  private
    fOwner : TwmMultiSyn;
    function GetItems(Index: integer): TgmScheme;
    procedure SetItems(Index: integer; const Value: TgmScheme);
{$IFDEF MWE_COMPILER_3_UP}
  protected
    function GetOwner: TPersistent; override;
{$ENDIF}
  public
    constructor Create( aOwner : TwmMultiSyn );
    property Items[Index:integer] : TgmScheme read GetItems write SetItems; default;
  end;

  TgmMarker = class
  protected
    fOwnerScheme : integer;
    fScheme : integer;
    fStartPos : integer;
    fMarkerLen: integer;
    fMarkerText : string;
  public
    constructor Create( aOwnerScheme, aScheme, aStartPos, aMarkerLen : integer;
                        aMarkerText : string );
  end;

  TwmMultiSyn = class(TmwCustomHighLighter)
  protected
    fSchemes    : TgmSchemes;
    fCurrScheme : integer;
    fLine       : string;
    fLine_tmp1  : string; //These Ref counts are getting me down :(
    fLine_tmp2  : string; //These Ref counts are getting me down :(
    FDefault    : TmwCustomHighLighter;
    fLineNumber : Integer;
    fRun        : Integer;
    fMarkers    : TList;
    fMarker     : TgmMarker;

    procedure SetSchemes(const Value: TgmSchemes);
    procedure ClearMarkers;
    function FindMarker( MarkerPos : integer ) : TgmMarker;
    function FindNextMarker( MarkerPos : integer ) : TgmMarker;
  public
    class function GetLanguageName: string; override;                           //gp 2000-01-20
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

//    procedure ExportNext; override;
    function GetDefaultAttribute(Index: integer): TmwHighLightAttributes;       //mh 2000-01-17
      override;
    function GetEol: Boolean; override;
    function GetRange: Pointer; override;
    function GetToken: String; override;
    function GetTokenAttribute: TmwHighLightAttributes; override;
    function GetTokenKind: integer; override;
    function GetTokenPos: Integer; override;
    procedure Next; override;
    procedure SetLine(NewValue: String; LineNumber:Integer); override;
//    procedure SetLineForExport(NewValue: String);override;
    procedure SetRange(Value: Pointer); override;
    procedure ReSetRange; override;
//    function UseUserSettings(settingIndex: integer): boolean; override;

    property CurrScheme : integer read fCurrScheme write fCurrScheme;
  published
    property Schemes : TgmSchemes read fSchemes write SetSchemes;
    property DefaultHighlighter: TmwCustomHighLighter read FDefault
      write FDefault;
  end;

procedure Register;

implementation

uses
  RegExpr, Graphics, SysUtils, mwLocalStr;

procedure Register;
begin
  RegisterComponents(MWS_ComponentsPage, [TwmMultiSyn]);                        //gp 2000-01-20
end;

function cmpMarker(Item1, Item2: Pointer): Integer;
begin
  if TgmMarker(Item1).fStartPos < TgmMarker(Item2).fStartPos then
    Result := -1
  else
    if TgmMarker(Item1).fStartPos > TgmMarker(Item2).fStartPos then
      Result := 1
    else
      Result := 0;
end;

function cmpMarkerPos(Item1: Pointer; Pos : integer): Integer;
begin
  if TgmMarker(Item1).fStartPos < Pos then
    Result := -1
  else
    if TgmMarker(Item1).fStartPos > Pos then
      Result := 1
    else
      Result := 0;
end;
{ TgmMarker }

constructor TgmMarker.Create(aOwnerScheme, aScheme, aStartPos, aMarkerLen: integer;
                        aMarkerText : string );
begin
  fOwnerScheme := aOwnerScheme; fScheme := aScheme; fStartPos := aStartPos;
  fMarkerLen := aMarkerLen; fMarkerText := aMarkerText;
end;

{ TwmMultiSyn }

procedure TwmMultiSyn.ClearMarkers;
var
  i : integer;
begin
  for i := 0 to fMarkers.Count-1 do
    TObject(fMarkers[i]).Free;
  fMarkers.Clear;
end;

constructor TwmMultiSyn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fSchemes := TgmSchemes.Create(Self);
  fCurrScheme := -1;
  fMarker     := nil;
  AddAttribute(TmwHighLightAttributes.Create(MWS_AttrMarker));
  with (GetAttribute(0)) do begin
    Background := clYellow;
    Style      := [fsBold];
  end;
  fMarkers := TList.Create;
end;

destructor TwmMultiSyn.Destroy;
begin
  fSchemes.Free;
  ClearMarkers;
  fMarkers.Free;
  inherited Destroy;
end;

function TwmMultiSyn.FindMarker(MarkerPos: integer): TgmMarker;
var
  L, H, I, C: Integer;
begin
  Result := nil;
  L := 0;
  H := fMarkers.Count - 1;
  while L <= H do begin
    I := (L + H) shr 1;
    C := cmpMarkerPos(fMarkers[I], MarkerPos);
    if C < 0 then
      L := I + 1
    else begin
      H := I - 1;
      if C = 0 then begin
        Result := fMarkers[I];
        break;
      end;
    end;
  end;
end;

function TwmMultiSyn.FindNextMarker(MarkerPos: integer): TgmMarker;
var
  L, H, I, C: Integer;
begin
  Result := nil;
  I := -1;
  L := 0;
  H := fMarkers.Count - 1;
  while L <= H do begin
    I := (L + H) shr 1;
    C := cmpMarkerPos(fMarkers[I], MarkerPos);
    if C < 0 then
      L := I + 1
    else begin
      H := I - 1;
      if C = 0 then
        break;
    end;
  end;
  if (I >= 0) then
    if (cmpMarkerPos(fMarkers[I], MarkerPos)>=0) then
      Result := fMarkers[I]
    else
      if I+1<fMarkers.Count then
        Result := fMarkers[I+1]
end;

{begin}                                                                         //mh 2000-01-17
function TwmMultiSyn.GetDefaultAttribute(Index: integer): TmwHighLightAttributes;
begin
  Result := nil;
end;
{end}                                                                           //mh 2000-01-17

function TwmMultiSyn.GetEol: Boolean;
begin
  if assigned(fMarker) then
    Result := false
  else
    if (fCurrScheme>=0) then
      Result := TgmScheme(fSchemes[fCurrScheme]).Highlighter.GetEol
    else
      if DefaultHighlighter<> nil then
        Result := DefaultHighlighter.GetEol
      else
        Result := true;
end;

class function TwmMultiSyn.GetLanguageName: string;                             //gp 2000-01-20
begin
  Result := MWS_LangGeneralMulti;
end;

function TwmMultiSyn.GetRange: Pointer;
var
  lResult : longint;
begin
  if (fCurrScheme<0) then
    if DefaultHighlighter<> nil then
      Result := DefaultHighlighter.GetRange
    else
      Result := nil
  else begin
    lResult := ((fCurrScheme+1) shl 16) or
            (integer(fSchemes[fCurrScheme].Highlighter.GetRange) and $0000FFFF);
    Result := pointer(lResult);
  end;
end;

function TwmMultiSyn.GetToken: String;
begin
  if assigned(fMarker) then
    Result := fMarker.fMarkerText
  else
    if (fCurrScheme>=0) then
      Result := fSchemes[fCurrScheme].Highlighter.GetToken
    else
      if DefaultHighlighter<> nil then
        Result := DefaultHighlighter.GetToken
      else
        Result := fLine;
end;

function TwmMultiSyn.GetTokenAttribute: TmwHighLightAttributes;
begin
  if assigned(fMarker) then
    Result := GetAttribute(0)
  else
    if (fCurrScheme>=0) then
      Result := fSchemes[fCurrScheme].Highlighter.GetTokenAttribute
    else
      if DefaultHighlighter<> nil then
        Result := DefaultHighlighter.GetTokenAttribute
      else
        Result := nil;
end;

function TwmMultiSyn.GetTokenKind: integer;
begin
  if (fCurrScheme>=0) then
    Result := fSchemes[fCurrScheme].Highlighter.GetTokenKind
  else
    if DefaultHighlighter<> nil then
      Result := DefaultHighlighter.GetTokenKind
    else
      Result := 0;
end;

function TwmMultiSyn.GetTokenPos: Integer;
begin
  Result := 0;
  if (fCurrScheme>=0) then
    Result := fRun + fSchemes[fCurrScheme].Highlighter.GetTokenPos -
              length(fSchemes[fCurrScheme].Highlighter.GetToken)
  else
    if DefaultHighlighter<> nil then
      Result := fRun + DefaultHighlighter.GetTokenPos -
                length(DefaultHighlighter.GetToken);
  if assigned(fMarker) then
    Result := fMarker.fStartPos-1;
end;

procedure TwmMultiSyn.Next;
var
  i, mx : integer;
  tok : string;
begin
  if DefaultHighlighter= nil then
    exit;

  fMarker := FindMarker( fRun + 1 );
  if assigned( fMarker ) then
    if (fCurrScheme <> fMarker.fScheme) and
       (((fMarker.fScheme<0)and(fMarker.fOwnerScheme=fCurrScheme)) or
        (fMarker.fScheme >=0) )then begin
      fCurrScheme := fMarker.fScheme;
      inc( fRun, fMarker.fMarkerLen );
      exit;
    end;
  fMarker := FindNextMarker( fRun + 1 );
  if assigned( fMarker ) and
     (fCurrScheme <> fMarker.fScheme) and
     (((fMarker.fScheme<0)and(fMarker.fOwnerScheme=fCurrScheme)) or
      (fMarker.fScheme >=0) )then
    mx := fMarker.fStartPos-1
  else
    mx := length(fLine);

  if (fCurrScheme<0) then begin
    fLine_tmp1 := copy(fLine, fRun+1, mx-fRun);
    DefaultHighlighter.SetLine( fLine_tmp1, fLineNumber );
    tok := DefaultHighlighter.GetToken;
  end
  else begin
    fLine_tmp1 := copy(fLine, fRun+1, mx-fRun);
    fSchemes[fCurrScheme].Highlighter.SetLine( fLine_tmp1, fLineNumber);
    tok := fSchemes[fCurrScheme].Highlighter.GetToken;
  end;
  fMarker := nil;
  for i := 1 to length(tok) do begin
    fMarker := FindMarker( fRun + i );
    if assigned( fMarker ) then begin
      if (fCurrScheme <> fMarker.fScheme) and
         (((fMarker.fScheme<0)and(fMarker.fOwnerScheme=fCurrScheme)) or
          (fMarker.fScheme >=0) )then
        fCurrScheme := fMarker.fScheme
      else
        fMarker := nil;
      break;
    end;
  end;
  if assigned(fMarker) then
    inc( fRun, fMarker.fMarkerLen )
  else
    inc( fRun, length(tok) );
end;

procedure TwmMultiSyn.ReSetRange;
begin
  fCurrScheme := -1;
end;

procedure TwmMultiSyn.SetLine(NewValue: String; LineNumber: Integer);
var
  i, j, k : integer;
  r    : TRegExpr;
  tmp  : string;
begin
  ClearMarkers;

  r := TRegExpr.Create;
  try
    for i := 0 to fSchemes.Count-1 do begin
      r.Expression := fSchemes[i].StartExpr;
      tmp := NewValue;
      k   := 0;
      while tmp <> '' do
        if r.exec(tmp) then
          for j := 0 to r.MatchCount-1 do begin
            fMarkers.Add( TgmMarker.Create(i, i, r.MatchPos[j]+k, r.MatchLen[j],
                          copy(tmp, r.MatchPos[j], r.MatchLen[j] )));
            delete( tmp, 1, r.MatchPos[j] + r.MatchLen[j] );
            inc(k, r.MatchPos[j] + r.MatchLen[j]);
          end
        else
          tmp := ''
    end;
    for i := 0 to fSchemes.Count-1 do begin
      r.Expression := fSchemes[i].EndExpr;
      tmp := NewValue;
      k   := 0;
      while tmp <> '' do
        if r.exec(tmp) then
          for j := 0 to r.MatchCount-1 do begin
            fMarkers.Add( TgmMarker.Create(i, -1, r.MatchPos[j]+k, r.MatchLen[j],
                          copy(tmp, r.MatchPos[j], r.MatchLen[j] )));
            delete( tmp, 1, r.MatchPos[j] + r.MatchLen[j] );
            inc(k, r.MatchPos[j] + r.MatchLen[j]);
          end
        else
          tmp := ''
    end;
  finally
    r.Free;
  end;

  fMarkers.Sort(cmpMarker);

  fLineNumber := LineNumber;
  fLine       := NewValue;

  fMarker     := nil;
  fRun        := 0;
  Next;
end;

procedure TwmMultiSyn.SetRange(Value: Pointer);
begin
  fCurrScheme := (Integer(Value) shr 16)-1;
  if DefaultHighlighter<> nil then begin
    if (fCurrScheme<0) then
      DefaultHighlighter.SetRange(pointer(Integer(Value) and $0000FFFF))
    else
      fSchemes[fCurrScheme].Highlighter.SetRange(pointer(Integer(Value) and $0000FFFF));
  end;
end;

procedure TwmMultiSyn.SetSchemes(const Value: TgmSchemes);
begin
  fSchemes.Assign(Value);
end;

{ TgmScheme }

procedure TgmScheme.SetEndExpr(const Value: string);
begin
  FEndExpr := Value;
end;

procedure TgmScheme.SetHighlighter(const Value: TmwCustomHighLighter);
begin
  FHighlighter := Value;
end;

procedure TgmScheme.SetStartExpr(const Value: string);
begin
  FStartExpr := Value;
end;

{ TgmSchemes }

constructor TgmSchemes.Create(aOwner: TwmMultiSyn);
begin
  inherited Create(TgmScheme);
  fOwner := aOwner;
end;

function TgmSchemes.GetItems(Index: integer): TgmScheme;
begin
  Result := inherited Items[Index] as TgmScheme;
end;

{$IFDEF MWE_COMPILER_3_UP}
function TgmSchemes.GetOwner: TPersistent;
begin
  Result := fOwner;
end;
{$ENDIF}

procedure TgmSchemes.SetItems(Index: integer; const Value: TgmScheme);
begin
  inherited Items[Index] := Value;
end;

end.
